/*
 * File: ChaosGame.java
 * ================================================================
 * An implementation of the Chaos Game.  The program picks three
 * points to act as vertices of a triangle (these will be the
 * upper-left, lower-right, and lower-left corners of the window).
 * Starting with the top point, we repeatedly do the following:
 * 
 * 1. Choose a random one of the three corners.
 * 2. Move halfway between our current location and that corner.
 * 3. Put a dot at our location.
 * 
 * The result is pretty surprising!
 */
import acm.graphics.*;
import acm.program.*;
import acm.util.*;

public class ChaosGame extends GraphicsProgram {
	public void run() {
		GPoint pt = new GPoint(0, 0);
		
		/* Repeatedly move toward a corner and plot a pixel. */
		while (true) {
			moveRandomly(pt);
			plotPixel(pt.getX(), pt.getY());
		}
	}
	
	/* Chooses a random corner and moves halfway toward it. */
	private void moveRandomly(GPoint pt) {
		GPoint dest = chooseRandomCorner();
		
		double newX = (pt.getX() + dest.getX()) / 2.0;
		double newY = (pt.getY() + dest.getY()) / 2.0;
		
		pt.setLocation(newX, newY);
	}
	
	/* Chooses and returns one of the three corners of the window at random. */
	private GPoint chooseRandomCorner() {
		RandomGenerator rgen = RandomGenerator.getInstance();
		int cornerIndex = rgen.nextInt(1, 3);
		
		if (cornerIndex == 1) {
			return new GPoint(0, 0);
		} else if (cornerIndex == 2) {
			return new GPoint(getWidth(), 0);
		} else {
			return new GPoint(0, getHeight());
		}
	}
	
	/* Draws a dot at the indicated location. */
	private void plotPixel(double x, double y) {
		GRect pixel = new GRect(x, y, 1, 1);
		pixel.setFilled(true);
		add(pixel);
	}
}




